package com.keysight.repository.example;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.time.Instant;
import java.time.ZoneOffset;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import com.keysight.repository.client.KeysightRepositoryClient;
import com.keysight.repository.model.Credentials;
import com.keysight.repository.model.Dataset;
import com.keysight.repository.model.Measurement;
import com.keysight.repository.model.MeasurementType;
import com.keysight.repository.model.PropertyType;
import com.keysight.repository.model.PropertyType.PropertyTypeFormat;
import com.keysight.repository.model.builder.MeasurementBuilder;

public class SimpleCreateAndFetch {

	private static final String DATASET_NAME = "Sample Dataset";
	private static final String MEAS_RAM_USAGE = "Ram Usage";
	private static final String PROP_CURRENT_TIME = "Current Time";
	private static final String PROP_SERVER_NAME = "Server Name";
	
	private static KeysightRepositoryClient client;
	private static Dataset dataset;
	private static MeasurementType measurementType;
	private static Map<String, PropertyType> propertyTypes;
	
	private static DateTimeFormatter ISO_8601 = DateTimeFormatter.ISO_INSTANT.withZone(ZoneOffset.UTC);
	private static DateTimeFormatter FRIENDLY_TIME = DateTimeFormatter.RFC_1123_DATE_TIME.withZone(ZoneOffset.systemDefault());
	
	private static void findOrCreateDataset() {
		List<Dataset> datasets = client.getDatasets();
		dataset = datasets.stream().filter(ds -> ds.getName().equals(DATASET_NAME)).findFirst().orElse(null);
		if(dataset == null) {
			dataset = new Dataset(DATASET_NAME);
			dataset = client.createDataset(dataset);
		}
	}
	
	private static void findOrCreateMeasurementType() {
		List<MeasurementType> measurementTypes = client.getMeasurementTypesForDataset(dataset.getId());
		measurementType = measurementTypes.stream().filter(mt -> mt.getName().equals(MEAS_RAM_USAGE)).findFirst().orElse(null);
		if(measurementType == null) {
			measurementType = new MeasurementType(dataset.getId(), MEAS_RAM_USAGE, "MB", 0.0, 1024.0*8);
			measurementType = client.createMeasurementTypes(dataset.getId(), Arrays.asList(measurementType)).get(0);
		}
	}
	
	private static PropertyType findOrCreateProperty(Map<String, PropertyType> propertyTypeMap, String propertyTypeName, Supplier<PropertyType> factory) {
		PropertyType result = propertyTypeMap.get(propertyTypeName);
		if(result == null) {
			result = factory.get();
			result = client.createPropertyTypes(dataset.getId(), Arrays.asList(result)).get(0);
		}
		return result;
	}
	
	private static void findOrCreatePropertyTypes() {
		propertyTypes = new HashMap<>();
		List<PropertyType> allPropertyTypes = client.getPropertyTypesForDataset(dataset.getId());
		Map<String, PropertyType> allPropertyTypesMap = allPropertyTypes.stream().collect(Collectors.toMap(pt -> pt.getName(), pt -> pt));
		propertyTypes.put(
				PROP_CURRENT_TIME,
				findOrCreateProperty(
						allPropertyTypesMap,
						PROP_CURRENT_TIME,
						() -> new PropertyType(PROP_CURRENT_TIME, PropertyTypeFormat.DateTime)));
		propertyTypes.put(PROP_SERVER_NAME,
				findOrCreateProperty(
						allPropertyTypesMap,
						PROP_SERVER_NAME,
						() -> new PropertyType(PROP_SERVER_NAME, PropertyTypeFormat.Text)));
	}
	
	private static String getServerName() {
		try {
			return InetAddress.getLocalHost().getHostName();
		} catch (UnknownHostException ex) {
			return "unknown";
		}
	}
	
	private static double bytesToMegabytes(double bytes) {
		return bytes / (1024 * 1024);
	}
	
	private static double computeRamUsage() {
		return bytesToMegabytes(Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory());
	}
	
	private static void createMeasurement() {
		double ramUsageInMb = computeRamUsage();
		Measurement measurement = MeasurementBuilder.newInstance(measurementType.getId(), ramUsageInMb)
				.addProperty(propertyTypes.get(PROP_CURRENT_TIME).getId(), ISO_8601.format(Instant.now()))
				.addProperty(propertyTypes.get(PROP_SERVER_NAME).getId(), getServerName())
				.build();
		client.createMeasurement(dataset.getId(), measurement);
	}
	
	private static String getProperty(Measurement measurement, String propertyName) {
		return measurement.getProperties().stream()
				.filter(prop -> prop.getPropertyTypeId() == propertyTypes.get(propertyName).getId())
				.findFirst()
				.orElseThrow(() -> new RuntimeException("Couldn't find property wtih name: " + propertyName))
				.getValue();
	}

	private static String getDate(Measurement measurement) {
		return FRIENDLY_TIME.format(ISO_8601.parse(getProperty(measurement, PROP_CURRENT_TIME), ZonedDateTime::from));
	}
	
	private static String getServerName(Measurement measurement) {
		return getProperty(measurement, PROP_SERVER_NAME);
	}
	
	private static void dumpMeasurementsToConsole() {
		for(Measurement measurement : client.getMeasurementsOfType(dataset.getId(), measurementType.getId())) {
			System.out.println(getDate(measurement) + ": <" + getServerName(measurement) + "> " + measurement.getValue());
		}
	}
	
	public static void main(String [] args) {
		client = KeysightRepositoryClient.open("localhost", 5000);
		client.login(new Credentials("admin", "admin"));
		findOrCreateDataset();
		findOrCreateMeasurementType();
		findOrCreatePropertyTypes();
		createMeasurement();
		dumpMeasurementsToConsole();
	}
	
}
