package com.keysight.repository.model;

/**
 * A measurement type defines a value that can be measured, for example Voltage or Current.  Measurement
 * values always have a measurement type.  The measurement type can have an upper or lower limit defined
 * by the user which is intended to represent reasonable values for a measurement.
 */
public class MeasurementType {

	private int id;
	private int datasetId;
	private String name;
	private String units;
	private Double lowerLimit;
	private Double upperLimit;
	
	protected MeasurementType() {
		
	}
	
	/**
	 * Constructs a measurement type with no upper or lower limit
	 * @param datasetId The id of the dataset for which the measurement is defined
	 * @param name The name of the measurement type
	 */
	public MeasurementType(int datasetId, String name, String units) {
		this(datasetId, name, units, null, null);
	}
	
	/**
	 * Constructs a measurement type
	 * @param datasetId The id of the dataset for which the measurement is defined
	 * @param name The name of the measurement type
	 * @param lowerLimit The lower limit of the measurement type or null if none is desired
	 * @param upperLimit The upper limit of the measurement type or null if none is desired
	 */
	public MeasurementType(int datasetId, String name, String units, Double lowerLimit, Double upperLimit) {
		this.datasetId = datasetId;
		this.name = name;
		this.units = units;
		this.lowerLimit = lowerLimit;
		this.upperLimit = upperLimit;
	}
	
	/**
	 * Returns the repository id of the measurement type.  This will be set by the
	 * repository server.
	 * @return The repository id of the measurement type.
	 */
	public int getId() {
		return id;
	}
	
	/**
	 * Returns the id of the dataset for which the measurement type is associated
	 * @return the id of the dataset for which the measurement type is associated
	 */
	public int getDatasetId() {
		return datasetId;
	}
	
	/**
	 * Returns the name of the measurement type
	 * @return The name of the measurement type
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Returns the units for the measurement type
	 * @return The units for the measurement type
	 */
	public String getUnits() {
		return units;
	}
	
	/**
	 * Returns the lower limit of the measurement type.  This is a defined by the user as a point
	 * at which measurements should not fall below.  It does not prevent measurements from existing
	 * with lower values.
	 * @return The lower limit or null if there is no lower limit
	 */
	public Double getLowerLimit() {
		return lowerLimit;
	}
	
	/**
	 * Returns the upper limit of the measurement type.  This is defined by the user as a point
	 * at which measurements should not rise above.  It does not prevent measurements from existing
	 * with higher values.
	 * @return The upper limit or null if there is no upper limit
	 */
	public Double getUpperLimit() {
		return upperLimit;
	}
	
}
