package com.keysight.repository.model;

/**
 * A property type represents a type of property that can be defined when a measurement is made.  Example
 * property types could include the date, the version of the device being tested, or the engineer running
 * the test.  Property types can have a number of different formats which define how the properties are
 * interpreted.
 * 
 * Property types may also be namespaced by a property group to represent where the property type's value
 * comes from.  For example, there may be two devices which have a property named size but the property
 * may mean different things.  In this case the group of the two property types would be different.
 */
public class PropertyType {

	/**
	 * The property type format restricts the values that a property can have as well as changes
	 * how the property type is interpreted.  For example, interpolation may be done when displaying
	 * a time based or numeric property type but would never be done on a freeform text property type.
	 * 
	 * Also, a numeric property would be sorted numerically while a text property would be sorted
	 * lexographically.
	 */
	public static enum PropertyTypeFormat {
		/**
		 * These properties represent an instant in time and should be ISO-8601 formatted strings
		 */
		DateTime,
		/**
		 * These properties represent decimal numbers (e.g. 3.456)
		 */
		Decimal,
		/**
		 * These properties represent integer numbers (e.g. 1, 3)
		 */
		Integer,
		/**
		 * These properties can by any string value desired
		 */
		Text,
		/**
		 * These properties represent dot separated versions (e.g. 1.2.3.4 or 10.307)
		 */
		Version,
		/**
		 * These properties represent true/false values
		 */
		Boolean
	}
	
	private int id;
	private String name;
	private String group;
	private PropertyTypeFormat format;
	
	protected PropertyType() {
		
	}

	/**
	 * Constructs a property type
	 * @param name The name of the property type (e.g. "Device Version")
	 * @param group The group of the property type (e.g. "Keysight")
	 * @param propertyTypeFormat The format of the property type
	 */
	public PropertyType(String name, String group, PropertyTypeFormat propertyTypeFormat) {
		this.name = name;
		this.format = propertyTypeFormat;
	}

	/**
	 * Constructs a property type in the default group
	 * @param name The name of the property type (e.g. "Device Version")
	 * @param propertyTypeFormat The format of the property type
	 */
	public PropertyType(String name, PropertyTypeFormat propertyTypeFormat) {
		this(name, null, propertyTypeFormat);
	}
	
	/**
	 * Constructs a property type with the freeform text format in the default group
	 * @param name The name of the property type (e.g. "Device Version")
	 */
	public PropertyType(String name) {
		this(name, null, PropertyTypeFormat.Text);
	}
	
	/**
	 * Returns the repository id of the property type.  This will be set by the repository
	 * server when the property type is created.
	 * @return The repository id of the property type
	 */
	public int getId() {
		return id;
	}
	
	/**
	 * Returns the property type's group
	 * @return The property type's group
	 */
	public String getGroup() {
		return group;
	}
	
	/**
	 * Returns the name of the property type
	 * @return The name of the property type
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Returns the property type format
	 * @return the property type format
	 */
	public PropertyTypeFormat getFormat() {
		return format;
	}
	
}
